/* 
 * Example of using Non-maskable break in MicroBlaze.
 */
#include <stdio.h>

#include "xparameters.h"
#include "xtmrctr.h"

static int break_count = 0;

/*
 * Declaration of break handler with attributes.
 *
 * Using a break handler requires adding the break vector manually in the linker
 * script and adding the call to the handler in assembler. See the files
 * lscript.ld and vectors.S in this example.
 */
void break_handler() __attribute__((break_handler));

/* Break handler.
 *
 * The handler just increments break count to show that it has been called.
 *
 */
void break_handler()
{
    break_count++;
}

int main()
{
    int Status;
    XTmrCtr TimerCounter; /* The instance of the timer counter */

    print("Entering main\r\n");

    Status = XTmrCtr_Initialize(&TimerCounter, XPAR_TMRCTR_0_DEVICE_ID);
    if (Status != XST_SUCCESS) {
        return XST_FAILURE;
    }

    /*
     * Set up timer to count down to zero and generate a one clock cycle
     * pulse on the generateout0 output to trigger a non-maskable break
     */
    XTmrCtr_SetOptions(&TimerCounter, 0, XTC_DOWN_COUNT_OPTION | XTC_EXT_COMPARE_OPTION);
    XTmrCtr_SetResetValue(&TimerCounter, 0, 0xFF);
    XTmrCtr_Start(&TimerCounter, 0);

    /* Wait until the timer has finished counting */
    while (! XTmrCtr_IsExpired(&TimerCounter, 0));

    /* Check that the break has occurred */
    if (break_count == 0) {
        print("Expected break did not occur\r\n");
        return XST_FAILURE;
    }

    print("Exiting main\r\n");

    return XST_SUCCESS;
}
